require 'benchmark'
# Automatically load all log analyzers (i.e. all files in lib/log_analyzers/ which end with _log_analyzer.rb)
Dir.glob($SCRIPT_DIR + '/lib/log_analyzers/*_log_analyzer.rb', &method(:require))

# This class serves as the parent class for the single hash functions (SHA1, SSDEEP, SDHASH).
# Its purpose is to initialize the common attributes and to hash files.
class BaseHashFunction
  
  # Parameters with which the program is executed.
  attr_accessor :params
  # Command = program name + params
  attr_accessor :command
  # Longest digest and the file on it was encountered. (Efficiency test)
  attr_accessor :max_digest_length
  # The complete output of the hash functions is stored here. The digest is extracted out of the output.
  attr_accessor :output
  # The time needed to execute an operation.
  attr_accessor :duration
  # A file, which contains every digest produced by the according hash function.
  attr_accessor :digest_file
  # Contains the statistics for every hased file. The stats are the filename, file size, digest and digest length.
  attr_accessor :statistics
  
  # Initializes the attributes.
  def initialize(params="")
    @params = params
    reset
  end
  
  def key
    self.class::PROGRAM_NAME.to_sym
  end
  # This function is called in the normal mode of the efficiency test. The input is passed directly to tha hash function, which is called only once.
  #
  # <i>IFS=$'n'</i> is bash's Internal Field Separator and is set to a newline here. This is needed to get files with spaces and other special characters working.
  def hash_fast(filenames, dirnames)
    find_cmd = dirnames.empty? ? "" : "`find #{dirnames.join(' ')} #{"-maxdepth #{$options.max_depth}" if $options.max_depth} -type f`"
#    print "BEFORE HASHING -- function: ", self.class::PROGRAM_NAME, ", duration = ", @duration.inspect, "\n"
    @duration[:total] = realtime {%x[IFS=$'\n'; #{@command} #{filenames.map{|f| '"'+f+'"'}.join(' ')} #{find_cmd} > #{$TMP_DIR+'/'+@digest_file[:name]} #{"2> /dev/null" unless $debug}]}
#    print "AFTER HASHING -- function: ", self.class::PROGRAM_NAME, ", duration = ", @duration.inspect, "\n"
    analyze_digests
  end
  
  # This function is called in the verbose mode of the efficiency test. Each hash functin is run once for every file.
  #
  # The other difference between this test and #hash_fast is that this one saves the statistics for every file, too.
  def hash_verbose(filename)
      duration = realtime { @output = %x[#{@command} "#{filename}" 2> /dev/null]}
      add_digest
      @duration[:file] = duration
      @duration[:total] += duration
  end
  
  # Every digest is written to a file. The digests are processed later by #analyze_digests.
  def write_digests_file
    File.open(@digest_file[:name], "w") do |f|
      f.write(@digest_header)
      @all_outputs.each { |line| f.write_nonblock(line.rstrip + "\n") unless line.empty? }
    end 
  end

  # The digests that were written to the digests file are processed and saved in the statistics attribute.
  #
  # Processed means, that the information is split to extract the file name, file size, digest and the digest length.
  # Furthermore, the maximum digest length is saved, as well as the file which produced the digest.
  def analyze_digests
    File.open(@digest_file[:name], 'r+') { |f| @digest_file[:lines] = f.readlines }
    @digest_file[:lines].shift if @digest_file[:lines].first == @digest_header
    @digest_file[:lines].each do |line|
      current_stats = self.class.file_stats(line)
      @statistics << current_stats
      @max_digest_length = current_stats if current_stats[:digest_length] > @max_digest_length[:digest_length]
    end
  end
  
  # The basic attributes are set to default values.
  def reset
    @command = self.class::PROGRAM_NAME + " " + @params
    @duration = { file: 0.0, total: 0.0, pairs: 0.0 }
    @output = []
    @all_outputs = []
    @statistics = []
    @digest_header = ""
    @digest_file = { file: nil, name: self.class::PROGRAM_NAME + "_digests.txt", lines: []}
    @max_digest_length = { filename: '', digest_length: 0 }
  end
  
  # This function is used in the verbose mode of the efficiency test. It adds every digest to a collections, which is later written to the digests file.
  #
  # As every file is hashed individually, the digests may contain a digest header, like ssdeep. This header is removed here before adding the digest to the collection.
  def add_digest
    @all_outputs << @output.gsub!(@digest_header, '') if @output != ""
  end
end
