require_relative 'fuzzy_hash_function'

# This class inherits from BaseHashFunction.
#
# It sets the \ssdeep program that will be used and computes the digest length after parsing the output of the program.
class SSDEEP < FuzzyHashFunction

  PROGRAM_NAME = "ssdeep"

  def initialize(params="")
    super(params)
    @digest_header = "ssdeep,1.1--blocksize:hash:hash,filename\n"
    @all_pair_comparison_command = PROGRAM_NAME + " -x"
    @gen_compare_command = PROGRAM_NAME + " -s -d"#" -s -a -d"
  end

  # Example digest
  #
  # 192:4pvKEqOTBObW17W/bMUVEjat6wVXjT3mS:4V+8nUCwHBjTmS,"/var/fileset/0000_10.odt"
  def self.file_stats(digest)
      info_hash = {}
      info_hash[:filename] = digest.split(/,"/)[-1].rstrip.gsub(/"$/, '') # "/var/fileset/0000_10.odt"
      info_hash[:filesize] = File.size info_hash[:filename]
      info_hash[:digest] = digest.split(',"')[0...-1].first # 192:4pvKEqOTBObW17W/bMUVEjat6wVXjT3mS:4V+8nUCwHBjTmS
      info_hash[:digest_length] = self.digest_length(info_hash[:digest])
      return info_hash    
  end

  # The digest consists of several parts, divided by a colon. The first part is a number, so every character counts as a byte.
  #
  # The following parts are base64-encoded. That is why they are split and their lengths are summed up.
  # Finally, to get the byte length, the sum of the base64 lengths are multiplied with 3/4.
  def self.digest_length(digest)
    return 0 if digest.nil? or digest.empty?
    parts = digest.split(':')
    base64_parts_length = parts[1..-1].inject(0) { |sum, b| sum + b.length }
    parts[0].length + (base64_parts_length * 3/4)
  end

  # Example match:
  #
  # /var/fileset/0000_15.odt matches /var/fileset/0000_10.odt (77)
  def self.split_match(match)
    parts = match.split(' ') # ["/var/fileset/0000_15.odt", "matches", "/var/fileset/0000_10.odt", "(77)"]
    left = File.basename(parts[0]).split('_')[0] # "0000"
    right = File.basename(parts[2]).split('_')[0] # "0000"
    score = parts[3][1...-1].to_i # 77
    [left,right,score]
  end
end