require_relative 'base_test'

class EfficiencyTest < BaseTest
  
  def initialize
    super
    @sha1 = SHA1.new
    @hash_functions.unshift(@sha1)
    @sdhash_p4 = SDHASH.new("-p 4")
    @hash_functions << @sdhash_p4
  end

  def test(filenames, dirnames, all_filenames)
    $verbose ? test_hashtime_and_compression_with_output(all_filenames) : test_hashtime_and_compression_without_output(filenames, dirnames.map{|d| '"'+d+'"'})
    test_all_pairs_comparison_time
    return true
  end
  
  def test_hashtime_and_compression_without_output(filenames, dirnames)
    @hash_functions.each do |h|
        h.hash_fast(filenames, dirnames)
        print as_status(("\n   " + h.command + " finished").ljust(25)), as_info(h.duration[:total].duration) if $debug
    end
  end
  
  def test_hashtime_and_compression_with_output(all_filenames)
    stats = {}
    stats[:functions] = @hash_functions.map { |h| h.command}
    all_filenames.each do |f|
      stats.merge!({filename: f, filesize: File.size(f)})
      @hash_functions.each do |h|
        h.hash_verbose(f)
        file_stats = h.class.file_stats(h.output)
        stats[h.command.to_sym] = { digest_length: file_stats[:digest_length], duration: h.duration[:file] }
      end   
      puts file_stats_table(stats), "\n"
    end
    @hash_functions.each do |h|
      h.write_digests_file
      h.analyze_digests
    end
  end
  
  def test_all_pairs_comparison_time
    @hash_functions.select{ |x| x.is_a? FuzzyHashFunction }.each do |h| 
      h.all_pairs_comparison
      print as_status(("\n   " + h.command + " finished").ljust(25)), as_info(h.duration[:pairs].duration) if $debug
    end
  end
  
  def results
    [runtime_efficiency_table, compression_table]
  end
  
  def runtime_efficiency_table
    rows = []
    sha1_time = 0.0
    @hash_functions.each do |h|
      sha1_time = h.duration[:total] if h.instance_of? SHA1
      avg_duration = h.duration[:total]/$file_count
      all_pairs_comparison_duration = h.is_a?(FuzzyHashFunction) ? precision(h.duration[:pairs]).to_s + " (~ #{h.duration[:pairs].duration})" : "-"
      rows << [
        as_bold(h.command),
        align_with_precision(avg_duration),
        align_with_precision(h.duration[:total]),
        align(all_pairs_comparison_duration),
        align_with_precision(h.duration[:total]/sha1_time, PRECISION[:low])
      ]
    end
    table = Terminal::Table.new({
      title: as_important_header("Runtime efficiency test result"),
      headings: ['', 'Average (sec)', 'Total (sec)', 'All-pairs-comp. (sec)', 'x SHA1 ^(-1)'].map { |h| as_header(h) },
      rows: rows,
      style: { border_x: '=', border_i: '.' }
    })
  end
  
  def compression_table
    rows = []
    processed_functions = []
    @hash_functions.each do |h|
      next if processed_functions.include? h.class::PROGRAM_NAME
      has_statistics = h.statistics.any?
      total_digest_length = h.statistics.inject(0) { |sum, h| sum + h[:digest_length] }
      max_dl = h.max_digest_length[:digest_length]
      max_dl_filename = h.max_digest_length[:filename]
      max_dl_filesize = max_dl_filename.empty? ? 0 : File.size(max_dl_filename)
      max_dl_ratio = (has_statistics and max_dl_filesize > 0) ? precision(100*max_dl/max_dl_filesize.to_f).to_s + " %" : "-"
      avg_digest_length = total_digest_length/$file_count
      avg_ratio = $avg_filesize == 0 ? "-" : precision(100*avg_digest_length/$avg_filesize.to_f)
      max_dl_with_ratio = (h.is_a?(FuzzyHashFunction) and has_statistics) ? to_human(max_dl) + " (#{max_dl_ratio})" : "-"
      max_dl_file_with_size = (h.is_a?(FuzzyHashFunction) and has_statistics) ? "'#{max_dl_filename}' #{to_human(max_dl_filesize)}" : "-"
      rows << [
        as_bold(h.command),
        align(to_human(avg_digest_length)),
        align(avg_ratio),
        align(max_dl_with_ratio),
        align(max_dl_file_with_size),
        align(to_human(File.size(h.digest_file[:name])))
      ]
      processed_functions << h.class::PROGRAM_NAME
    end
    table = Terminal::Table.new({
      title: as_important_header("Compression test result (The file name and its size have no influence)"),
      headings: ['', 'Avg. digest length', 'Avg. ratio (%)', 'Max. digest length (ratio)', 'Max. digest length file (Size)', 'Digest file size'].map { |h| as_header(h) },
      rows: rows,
      style: { border_x: '=', border_i: '.' }
    })
  end

  def file_stats_table(stats)
    rows = []
    stats[:functions].each do |h|
      rows << [
        h,
        to_human(stats[h.to_sym][:digest_length]),
        stats[:filesize] > 0 ? (100*stats[h.to_sym][:digest_length]/stats[:filesize].to_f).round(5).to_s + " %" : "-",
        stats[h.to_sym][:duration].round(3).to_s + " s (~ #{stats[h.to_sym][:duration].duration})"
      ]
    end
    table = Terminal::Table.new({
      title: as_header("File stats for '#{stats[:filename]}' (Size: #{to_human(stats[:filesize])})"),
      headings: ['', 'Digest length', 'Ratio', 'Hash time'],
      rows: rows, 
      style: { border_x: '=', border_i: '.' }
    })
  end
  
  
  
  private

  def tab_header
    "Command".ljust(PAD_SPACES) + "Hash length".ljust(PAD_SPACES) + "Ratio".ljust(PAD_SPACES) + "Time\n"
  end
  
  def file_with_size(file)
    filesize = File.size(file)
    "File: <#{file}>  ~  Size: #{to_human(filesize)}"
  end
  
  def h_line
    "___________________________________________________________________________________________\n"
  end
  
  def print_file_stats(h, file)
    dl = h.digest_length
    file_size = File.size(file)
    ratio = ((dl/file_size.to_f)*100.0).round(5).to_s + "%"
    print h.command.ljust(PAD_SPACES), dl.to_s.ljust(PAD_SPACES), ratio.ljust(PAD_SPACES)
    print h.duration[:file]
    puts
  end  
  
end 